#======================================================================
#
#  PROGRAM NAME:  LEDtableV1.1.py
#
#======================================================================
#  DESCRIPTION: Graphical User Interface for
#               the Mini LED table project
#======================================================================
#  REQU. LIBRARIES: Tinker, OpenCV
#
#======================================================================
# Author:        DaniloR29
# Version:       1.1
# Copyright (C): 2022-2023 Danilo29R
# Distributed under terms of the MIT license.
#======================================================================

import tkinter as tk
import cv2
import picamera
import numpy as np
from PIL import Image, ImageTk

# Create a new Tkinter window
root = tk.Tk()
root.title("Camera Viewer")

# Create a new PiCamera object
camera = picamera.PiCamera()

# Set camera resolution
camera.resolution = (1280, 720)
camera.framerate = 20
camera.awb_mode = 'auto'
camera.awb_gains = 8
camera.exposure_mode = 'auto'

# Create a new canvas to display the camera output
canvas = tk.Canvas(root, width=camera.resolution[0], height=camera.resolution[1],bg="pink")
canvas.grid(row=0,column=0,sticky="NWSE",columnspan=5)

# Create a function to update the camera output on the canvas
def update_image():
    # Capture a new image from the camera
    camera.capture('temp.jpg')
    # Open the captured image using PIL
    image = Image.open('temp.jpg')
    # Resize the image to fit the canvas
    image = image.resize((camera.resolution[0], camera.resolution[1]))

    # Convert the PIL image to a Tkinter-compatible format
    photo = ImageTk.PhotoImage(image)

    # Update the canvas with the new image
    canvas.create_image(0, 0, anchor=tk.NW, image=photo)

# Add the red rectangle on top of the image
    rect_width =  camera.resolution[0]-2*x_thickness
    rect_height = camera.resolution[1]-2*y_thickness
    if (fOval):

        canvas.create_oval(x_thickness, y_thickness, x_thickness+rect_width, y_thickness+rect_height, outline='red', width=2)
        canvas.create_rectangle(x_thickness, y_thickness, x_thickness+rect_width, y_thickness+rect_height, outline='red', width=2)

        canvas.create_line(x_thickness, camera.resolution[1]*0.5 , x_thickness+rect_width, camera.resolution[1]*0.5, width=1,dash=(5,1), fill="red")
        canvas.create_line(camera.resolution[0]*0.5, y_thickness, camera.resolution[0]*0.5, y_thickness+rect_height, width=1, dash=(5,1), fill="red")

    # Keep a reference to the photo to prevent it from being garbage collected
    canvas.photo = photo
    # Schedule the function to run again after a short delay (30ms)
    if (fVideo):
        root.after(30, update_image)

def Quit():
    exit()

# Create a function to capture a photo and save it to a file with a user-defined name
def capture_photo():
    filename = filename_entry.get()
    if filename:
        camera.capture(filename)
    else:
        camera.capture('photo.jpg')

def analyse_photo():
    # Load the image
    image = cv2.imread('temp.jpg')

    height, width=image.shape[:2]

# Create a mask to remove the border of the image from the analysis
    mask = np.zeros(image.shape[:2], dtype="uint8")

# Preprocess the image
    cv2.rectangle(mask,(100,1),(1200,720),255,-1)
    img = cv2.bitwise_and(image, image, mask=mask)

    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    gray = cv2.medianBlur(gray, 5)

# remove the contours from the image and show the resulting images
    gray_mask = cv2.bitwise_and(gray, gray, mask=mask)

# Detect circles
    circles = cv2.HoughCircles(gray_mask, cv2.HOUGH_GRADIENT, 1, 20, param1=50, param2=20, minRadius=3, maxRadius=20)

# Count the pearls
    num_pearls = len(circles[0])

# Detect circles

# Classify the pearls and highlight them on the image
    nc=0
    for i in circles[0]:
        # Crop the area around the pearl
        x = int(i[0]-i[2])
        y = int(i[1]-i[2])
        w = int(i[2]*2)
        h = int(i[2]*2)
        pearl = img[y:y+h, x:x+w]

   # Draw a circle around the pearl and label its color
        cv2.circle(img, (int(i[0]), int(i[1])), int(i[2]), (255, 255, 255), 2)

# Output the number of pearls and rods
    cv2.putText(img, "Number of pearls: " + str(num_pearls), (50, 50), cv2.FONT_HERSHEY_SIMPLEX, 1, (255, 255, 255), 2)

    # Convert the OpenCV image to a PIL image
    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    pil_img = Image.fromarray(img)

    # Resize the image to fit the canvas
    pil_img = pil_img.resize((camera.resolution[0], camera.resolution[1]))

    # Convert the PIL image to a Tkinter-compatible format
    photo = ImageTk.PhotoImage(pil_img)

    # Update the canvas with the new image
    canvas.create_image(0, 0, anchor=tk.NW, image=photo)

    # Keep a reference to the photo to prevent it from being garbage collected
    canvas.photo = photo

def update_slider_values():
    global x_thickness, y_thickness,fVideo
    fVideo=0
    camera.brightness = brightness_slider.get()
    camera.contrast = contrast_slider.get()
    camera.saturation = saturation_slider.get()
    x_thickness = xborder_slider.get()
    y_thickness = yborder_slider.get()
    update_image()

def VideoMod():
    global fVideo
    if (fVideo==0):
        fVideo=1
    else:
        fVideo=0
    update_image()

def ShapeMod():
    global fOval
    if (fOval==0):
        fOval=1
    else:
        fOval=0
    update_image()


# configure the grid
root.columnconfigure(0, weight=1)
root.columnconfigure(1, weight=3)
root.columnconfigure(2, weight=3)
root.columnconfigure(3, weight=3)
root.columnconfigure(4, weight=3)
root.columnconfigure(5, weight=3)

fVideo=0
fOval=0

# Create sliders to adjust camera parameters
brightness_label = tk.Label(root, text="Brightness:",bg="cyan")
brightness_label.grid(row=1, column=0, sticky='NSWE')

brightness_slider = tk.Scale(root, from_=0, to=100, orient=tk.HORIZONTAL, resolution=1, command=lambda x: update_slider_values(),bg="pink")
brightness_slider.set(50)
brightness_slider.grid(row=1, column=1, sticky='NSWE')

contrast_label = tk.Label(root, text="Contrast:",bg="cyan")
contrast_label.grid(row=2, column=0, sticky='NSWE')

contrast_slider = tk.Scale(root, from_=-100, to=100, orient=tk.HORIZONTAL, resolution=1, command=lambda x : update_slider_values(),bg="pink")
contrast_slider.set(0)
contrast_slider.grid(row=2, column=1, sticky='NSWE')

saturation_label = tk.Label(root, text="Saturation:",bg="cyan")
saturation_label.grid(row=3, column=0, sticky='NSWE')

saturation_slider = tk.Scale(root, from_=-100, to=100, orient=tk.HORIZONTAL, resolution=1, command=lambda x :update_slider_values(),bg="pink")
saturation_slider.set(0)
saturation_slider.grid(row=3, column=1, sticky='NSWE')

# Create a button to capture a photo
capture_button = tk.Button(root, text="Capture", command=capture_photo,bg="yellow")
capture_button.grid(row=4, column=2, sticky='NSWE')
analyse_button = tk.Button(root, text="Counting", command=analyse_photo,bg="orange")
analyse_button.grid(row=4, column=3, sticky='NSWE')

quit = tk.Button(root, command=Quit, text="Quit", bg="brown")
quit.grid(row=4, column=4, sticky="NSWE")

SetShape = tk.Button(root, command=ShapeMod, text="Marking Lines", bg="red")
SetShape.grid(row=1, column=4, sticky="NSWE")


refresh = tk.Button(root, command=update_image, text="Refresh", bg="green")
refresh.grid(row=3, column=4, sticky="NSWE")
contMod = tk.Button(root, command=VideoMod, text="VideoMode", bg="purple")
contMod.grid(row=2, column=4, sticky="NSWE")

# Create a label and text entry for the filename
filename_label = tk.Label(root, text="Filename:",bg="Yellow")
filename_label.grid(row=4,column=0,sticky='NSEW')

filename_entry = tk.Entry(root,textvariable="photo.jpg",bg="white")
filename_entry.insert(-1,"photo.jpg")
filename_entry.grid(row=4,column=1,sticky='NSWE')

x_thickness=100
y_thickness=100
xborder_label = tk.Label(root, text="Xborder:",bg="blue")
xborder_label.grid(row=2, column=2, sticky='NSWE')

xborder_slider = tk.Scale(root, from_=0, to=300, orient=tk.HORIZONTAL, resolution=1, command=lambda x : update_slider_values(),bg="orange")
xborder_slider.set(x_thickness)
xborder_slider.grid(row=2, column=3, sticky='NSWE')

yborder_label = tk.Label(root, text="Yborder:",bg="blue")
yborder_label.grid(row=3, column=2, sticky='NSWE')

yborder_slider = tk.Scale(root, from_=0, to=300, orient=tk.HORIZONTAL, resolution=1, command= lambda x: update_slider_values(),bg="orange")
yborder_slider.set(y_thickness)
yborder_slider.grid(row=3, column=3, sticky='NSWE')

# Start the camera output update loop
update_image()

# Start the Tkinter main loop
root.mainloop()

